==> SimpleMailer-0.90.tar.bz2: SimpleMailer-0.90/core/Smdn.Net.Pop3.Client/Smdn.Net.Pop3.Protocol.Client/PopConnection.cs <==
// 
// Author:
//       smdn <smdn@smdn.jp>
// 
// Copyright (c) 2008-2013 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Net;
using System.Net.Security;
using System.Net.Sockets;
using System.Security.Authentication;
using System.Security.Cryptography.X509Certificates;

namespace Smdn.Net.Pop3.Protocol.Client {
  public sealed class PopConnection : PopConnectionBase {
    /*
     * class members
     */
    [Obsolete("use Smdn.Net.Pop3.Client.PopSslConnection.ClientCertificates", true)]
    public static X509Certificate2Collection ClientCertificates {
      get { throw new NotSupportedException(); }
    }

    [Obsolete("use Smdn.Net.Pop3.Client.PopSslConnection.ServerCertificateValidationCallback", true)]
    public static RemoteCertificateValidationCallback ServerCertificateValidationCallback {
      get { throw new NotSupportedException(); }
      set { throw new NotSupportedException(); }
    }

    [Obsolete("use Smdn.Net.Pop3.Client.PopSslConnection.ClientCertificateSelectionCallback", true)]
    public static LocalCertificateSelectionCallback ClientCertificateSelectionCallback {
      get { throw new NotSupportedException(); }
      set { throw new NotSupportedException(); }
    }

    [Obsolete("use Smdn.Net.Pop3.Client.PopSslConnection or custom delegates", true)]
    public static Stream CreateSslStream(ConnectionBase connection, Stream baseStream)
    {
      throw new NotSupportedException();
    }

    public static TraceSource TraceSource {
      get { return popConnectionTraceSource; }
    }

    private static readonly TraceSource popConnectionTraceSource = new TraceSource("POP");

    /*
     * properties
     */

    // for autologout timer
    public DateTime LastSentTime {
      get { return lastSentTime; }
    }

    internal bool HandleResponseAsMultiline {
      get { return (Receiver as PopResponseReceiver).HandleAsMultiline; }
      set { (Receiver as PopResponseReceiver).HandleAsMultiline = value; }
    }

    public PopConnection(string host,
                         int port,
                         int millisecondsTimeout,
                         UpgradeConnectionStreamCallback createAuthenticatedStreamCallback)
      : base(popConnectionTraceSource)
    {
      if (port == -1) {
        if (createAuthenticatedStreamCallback == null)
          port = PopDefaultPorts.Pop;
        else
          port = PopDefaultPorts.Pops;
      }

      Connect(host,
              port,
              millisecondsTimeout,
              createAuthenticatedStreamCallback);
    }

    internal void SetIsSecureConnection(bool isSecureConnection)
    {
      base.IsSecureConnection = isSecureConnection;
    }

    protected override PopSender CreateSender(LineOrientedBufferedStream stream)
    {
      return new PopCommandSender(stream);
    }

    protected override PopReceiver CreateReceiver(LineOrientedBufferedStream stream)
    {
      return new PopResponseReceiver(stream);
    }

#region "seinding / receiving"
    public void SendCommand(PopCommand command)
    {
      CheckConnected();

      try {
        (Sender as PopCommandSender).Send(command);

        lastSentTime = DateTime.Now;
      }
      catch (SocketException ex) {
        throw CreateSocketErrorException(ex, false);
      }
      catch (IOException ex) {
        var socketException = ex.InnerException as SocketException;

        if (socketException == null)
          // unexpected IO error
          throw;
        else
          throw CreateSocketErrorException(socketException, false);
      }
    }

    public PopResponse TryReceiveResponse()
    {
      CheckConnected();

      try {
        // receive and parse response
        return (Receiver as PopResponseReceiver).ReceiveResponse();
      }
      catch (SocketException ex) {
        throw CreateSocketErrorException(ex, true);
      }
      catch (IOException ex) {
        var socketException = ex.InnerException as SocketException;

        if (socketException == null)
          // unexpected IO error
          throw;
        else
          throw CreateSocketErrorException(socketException, true);
      }
    }

    public bool TryReceiveLine(Stream stream)
    {
      CheckConnected();

      try {
        return (Receiver as PopResponseReceiver).ReceiveLine(stream);
      }
      catch (IOException ex) {
        var socketException = ex.InnerException as SocketException;

        if (socketException == null)
          // unexpected IO error
          throw;
        else
          throw CreateSocketErrorException(socketException, true);
      }
    }

    private Exception CreateSocketErrorException(SocketException ex, bool receiving)
    {
      switch (ex.SocketErrorCode) {
        case SocketError.WouldBlock:
        case SocketError.TimedOut:
          // timeout error
          return new TimeoutException(receiving ? "receive timeout" : "send timeout", ex);

        default:
          // unexpected socket exception
          return new PopConnectionException(string.Format("unexpected socket error ({0})", ex.SocketErrorCode),
                                            ex);
      }
    }
#endregion

    private DateTime lastSentTime = DateTime.Now;
  }
}

==> SimpleMailer-1.02.tar.bz2: SimpleMailer-1.02/core/Smdn.Net.Pop3.Client/Smdn.Net.Pop3.Protocol.Client/PopConnection.cs <==
// 
// Author:
//       smdn <smdn@smdn.jp>
// 
// Copyright (c) 2008-2014 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Net;
using System.Net.Security;
using System.Net.Sockets;
using System.Security.Authentication;
using System.Security.Cryptography.X509Certificates;

namespace Smdn.Net.Pop3.Protocol.Client {
  public sealed class PopConnection : ConnectionBase {
    /*
     * class members
     */
    public static TraceSource TraceSource {
      get { return popConnectionTraceSource; }
    }

    private static readonly TraceSource popConnectionTraceSource = new TraceSource("POP");

    /*
     * properties
     */

    // for autologout timer
    public DateTime LastSentTime {
      get { return lastSentTime; }
    }

    internal bool HandleResponseAsMultiline {
      get { return receiver.HandleAsMultiline; }
      set { receiver.HandleAsMultiline = value; }
    }

    public PopConnection(string host,
                         int port,
                         int millisecondsTimeout,
                         UpgradeConnectionStreamCallback createAuthenticatedStreamCallback)
      : base(popConnectionTraceSource)
    {
      if (port == -1) {
        if (createAuthenticatedStreamCallback == null)
          port = PopDefaultPorts.Pop;
        else
          port = PopDefaultPorts.Pops;
      }

      Connect(host,
              port,
              millisecondsTimeout,
              createAuthenticatedStreamCallback);
    }

    internal void SetIsSecureConnection(bool isSecureConnection)
    {
      base.IsSecureConnection = isSecureConnection;
    }

    protected override LineOrientedBufferedStream CreateBufferedStream(Stream stream)
    {
      var bufferedStream = base.CreateBufferedStream(stream);

      receiver  = new PopResponseReceiver(bufferedStream);
      sender    = new PopCommandSender(bufferedStream);

      return bufferedStream;
    }

    protected override Exception CreateConnectException(string message, Exception innerException)
    {
      return new PopConnectionException(message, innerException);
    }

    protected override Exception CreateUpgradeStreamException(string message, Exception innerException)
    {
      return new PopUpgradeConnectionException(message, innerException);
    }

#region "seinding / receiving"
    public void SendCommand(PopCommand command)
    {
      CheckConnected();

      try {
        sender.Send(command);

        lastSentTime = DateTime.Now;
      }
      catch (SocketException ex) {
        throw CreateSocketErrorException(ex, false);
      }
      catch (IOException ex) {
        var socketException = ex.InnerException as SocketException;

        if (socketException == null)
          // unexpected IO error
          throw;
        else
          throw CreateSocketErrorException(socketException, false);
      }
    }

    public PopResponse TryReceiveResponse()
    {
      CheckConnected();

      try {
        // receive and parse response
        return receiver.ReceiveResponse();
      }
      catch (SocketException ex) {
        throw CreateSocketErrorException(ex, true);
      }
      catch (IOException ex) {
        var socketException = ex.InnerException as SocketException;

        if (socketException == null)
          // unexpected IO error
          throw;
        else
          throw CreateSocketErrorException(socketException, true);
      }
    }

    public bool TryReceiveLine(Stream stream)
    {
      CheckConnected();

      try {
        return receiver.ReceiveLine(stream);
      }
      catch (IOException ex) {
        var socketException = ex.InnerException as SocketException;

        if (socketException == null)
          // unexpected IO error
          throw;
        else
          throw CreateSocketErrorException(socketException, true);
      }
    }

    private Exception CreateSocketErrorException(SocketException ex, bool receiving)
    {
      switch (ex.SocketErrorCode) {
        case SocketError.WouldBlock:
        case SocketError.TimedOut:
          // timeout error
          return new TimeoutException(receiving ? "receive timeout" : "send timeout", ex);

        default:
          // unexpected socket exception
          return new PopConnectionException(string.Format("unexpected socket error ({0})", ex.SocketErrorCode),
                                            ex);
      }
    }
#endregion

    private DateTime lastSentTime = DateTime.Now;
    private PopResponseReceiver receiver = null;
    private PopCommandSender sender = null;
  }
}
